package org.fouram.service;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.fouram.constants.WebConstants;
import org.fouram.core.base.model.PageData;
import org.fouram.core.base.service.BaseService;
import org.fouram.core.base.service.RequestService;
import org.fouram.core.plugin.weixin.cp.util.WXCpUserUtil;
import org.fouram.core.util.Base64Util;
import org.fouram.core.util.ConfConfig;
import org.fouram.core.util.JSONUtil;
import org.fouram.core.util.JsonNodeUtil;
import org.fouram.core.util.Tools;
import org.fouram.dto.GetToBeConfirmedListDTO;
import org.fouram.entity.SapOrg;
import org.fouram.entity.SapUser;
import org.fouram.entity.WxUser;
import org.fouram.enums.DelFlagEnum;
import org.fouram.enums.SapUserResultEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;

import me.chanjar.weixin.cp.bean.WxCpUser;

@Service
public class SapUserService extends BaseService {

	@Autowired
	private SapOrgService sapOrgService;
	@Autowired
	private WxUserService wxUserService;
	@Autowired
	private RequestService requestService;
	@Autowired
	private SapEmpJobService sapEmpJobService;
	@Autowired
	private SapOprLogService sapOprLogService;
	@Autowired
	private SapPersonalService sapPersonalService;
	@Autowired
	private SapBlacklistService sapBlacklistService;

	public void saveOrUpdateSapUsers() throws Exception {
		HttpHeaders headers = getHttpHeaders();
		String userUrl = "https://api15.sapsf.cn/odata/v2/User?$filter=status in 'active'&$format=json&$expand=personKeyNav";
		// get
		JsonNode node = requestService.getForObject(userUrl, headers);
		List<SapUser> sapUsers = Lists.newArrayList();
		StringBuffer name = null;
		if (node != null) {
			Iterator<JsonNode> iterator = node.get("d").get("results").elements();
			while (iterator.hasNext()) {
				JsonNode userNode = iterator.next();
				String userId = JsonNodeUtil.getValue(userNode, "userId");
				// 去除非员工账号和测试数据
				if (!userId.startsWith("Test")) {
					String username = JsonNodeUtil.getValue(userNode, "username");
					name = new StringBuffer().append(JsonNodeUtil.getValue(userNode, "lastName"))
							.append(JsonNodeUtil.getValue(userNode, "firstName"));
					String mobile = JsonNodeUtil.getValue(userNode, "cellPhone");
					String businessPhone = JsonNodeUtil.getValue(userNode, "businessPhone");
					String email = JsonNodeUtil.getValue(userNode, "email");
					String personId = JsonNodeUtil.getValue(userNode, "personKeyNav", "personIdExternal");
					SapUser sapUser = SapUser.builder().userId(userId).personId(personId).username(username)
							.name(name.toString()).mobile(mobile).formatMobile(mobile).businessPhone(businessPhone)
							.formatBusinessPhone(formatBusinessPhone(businessPhone)).email(email).formatEmail(email)
							.isPrimary(userId.equals(personId)).build();
					sapUsers.add(sapUser);
				}
			}
		}
		// 获取用户对应机构
		sapOrgService.setOrg(sapUsers);
		// 获取用户对应性别
		sapPersonalService.setGender(sapUsers);
		// 获取用户对应职务
		sapEmpJobService.setPosition(sapUsers);
		// 更新用户
		SapUser sapUserDb = null;
		SapUser sapUser = null;
		for (int i = 0; i < sapUsers.size(); i++) {
			sapUser = sapUsers.get(i);
			sapUser.setSortNumber(i + 1);
			sapUserDb = this.selectByUserId(sapUser.getUserId());
			if (sapUserDb == null) {
				this.saveSapUser(sapUser);
			} else {
				if (sapUserDb.toString().equals(sapUser.toString())) {
					// 用户数据没变，result保留SUCCESS，不用重复同步
					this.updateDelFlag(sapUser.getUserId(), DelFlagEnum.ENABLE);
				} else {
					this.updateSapUser(sapUser);
				}
			}
		}
	}

	/**
	 * 更新异常情况，敏感数据、入职
	 * 
	 * @throws Exception
	 */
	public void updateAbnormalResult() throws Exception {
		List<SapUser> normalList = selectPrimaryListByResult(SapUserResultEnum.NORMAL.getCode());
		SapUserResultEnum resultEnum = null;
		for (SapUser sapUser : normalList) {
			resultEnum = checkSensitiveOrEntryData(sapUser);
			if (resultEnum != null) {
				sapUser.setResult(resultEnum.getCode());
				this.updateResult(sapUser);
			}
		}
	}

	private SapUserResultEnum checkSensitiveOrEntryData(SapUser sapUser) throws Exception {
		WxUser wxUser = wxUserService.selectById(sapUser.getPersonId());
		if (wxUser == null) {
			return SapUserResultEnum.ENTRY;
		} else if (!wxUser.getPosition().equals(sapUser.getPosition()) || !wxUserService
				.getFullDepartName(sapUser.getPersonId()).equals(getFullOrgName(sapUser.getPersonId()))) {
			return SapUserResultEnum.SENSITIVE;
		}
		return null;
	}

	/**
	 * 根据personId查询多个用户完整的机构名称信息
	 * @param personId
	 * @return
	 * @throws Exception
	 */
	public String getFullOrgName(String personId) throws Exception {
		List<SapUser> users = selectListByPersonId(personId);
		StringBuffer sb = new StringBuffer();
		for (SapUser user : users) {
			if(sb.indexOf(user.getFullOrgName()) < 0) {
				sb.append(user.getFullOrgName()).append(";");
			}
		}
		return sb.substring(0, sb.length() - 1);
	}

	public void saveSapUser(SapUser entity) throws Exception {
		entity.setCreateDate(new Date());
		entity.setResult(SapUserResultEnum.NORMAL.getCode());
		entity.setDelFlag(DelFlagEnum.ENABLE.getCode());
		save("SapUserMapper.saveOne", entity);
	}

	public void updateSapUser(SapUser entity) throws Exception {
		entity.setResult(SapUserResultEnum.NORMAL.getCode());
		entity.setDelFlag(DelFlagEnum.ENABLE.getCode());
		update("SapUserMapper.updateOne", entity);
	}

	public void updateSyncWxResult(String personId) throws Exception {
		String result = createOrUpdateWXUser(personId);
		SapUser entity = SapUser.builder().personId(personId).result(result).build();
		update("SapUserMapper.updateResult", entity);
	}

	private String createOrUpdateWXUser(String personId) throws Exception {
		List<SapUser> sapUsers = selectListByPersonId(personId);
		SapUser primarySapUser = sapUsers.get(0);
		String mobile = primarySapUser.getFormatBusinessPhone();
		// 测试企业微信，手机号码脱敏
		// fouram wxCp.corpId = ww51602aeb8dab7a95
		// prod wxCp.corpId = ww911e29458d3a46fd
		if (!ConfConfig.getConfigString("wxCp.corpId").equals("ww911e29458d3a46fd")) {
			if (StringUtils.isNotBlank(mobile) && mobile.length() == 11) {
				mobile = mobile.substring(0, 8) + "999";
			} else {
				mobile = "";
			}
		}
		Long[] firstWxDepartIdList = new Long[sapUsers.size()];
		Long[] secondWxDepartIdList = new Long[sapUsers.size()];
		Long[] thirdWxDepartIdList = new Long[sapUsers.size()];
		Long[] fourthWxDepartIdList = new Long[sapUsers.size()];
		for (int i = 0; i < sapUsers.size(); i++) {
			SapUser sapUser = sapUsers.get(i);
			String checkResult = checkSapUser(sapUser);
			if (StringUtils.isNotBlank(checkResult)) {
				return checkResult;
			}
			firstWxDepartIdList[i] = sapUser.getFirstWxDepartId();
			secondWxDepartIdList[i] = sapUser.getSecondWxDepartId();
			thirdWxDepartIdList[i] = sapUser.getThirdWxDepartId();
			fourthWxDepartIdList[i] = sapUser.getFourthWxDepartId();
		}
		return WXCpUserUtil.createOrUpdate(personId, primarySapUser.getName(), firstWxDepartIdList,
				secondWxDepartIdList, thirdWxDepartIdList, fourthWxDepartIdList, primarySapUser.getFormatPosition(),
				primarySapUser.getFormatMobile(), primarySapUser.getGender(), primarySapUser.getUsername(), mobile,
				primarySapUser.getFirstOrgName());
	}

	/**
	 * 判断是不是垃圾数据或者黑名单
	 * 
	 * @param sapUser
	 * @return
	 * @throws Exception
	 */
	public String checkSapUser(SapUser sapUser) throws Exception {
		if (Tools.isEmpty(sapUser.getPersonId()) || Tools.isEmpty(sapUser.getFirstOrgName())
				|| WebConstants.INGORE_PERSONIDS.contains("," + sapUser.getPersonId() + ",")) {
			return SapUserResultEnum.GARBAGE.getCode();
		} else if (sapBlacklistService.findByUserId(sapUser.getPersonId()) != null) {
			return SapUserResultEnum.BLACKLIST.getCode();
		}
		return null;
	}

	public SapUser selectByUserId(String userId) throws Exception {
		return (SapUser) findObject("SapUserMapper.selectByUserId", userId);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectListByPersonId(String personId) throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectListByPersonId", personId);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectUnSyncList() throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectUnSyncList", null);
	}

	public void updateAllDelFlag(Integer delFlag) throws Exception {
		update("SapUserMapper.updateAllDelFlag", delFlag);
	}

	public void updateDelFlag(String userId, DelFlagEnum delFlagEnum) throws Exception {
		SapUser entity = SapUser.builder().userId(userId).delFlag(delFlagEnum.getCode()).build();
		update("SapUserMapper.updateDelFlag", entity);
	}

	public static void main(String[] args) {
		StringBuilder stringBuilder = new StringBuilder();
		String sapApiUser = "API01@cmccoltd:cmccoltd123";
		String authorizationStr = stringBuilder.append("Basic ").append(Base64Util.encodeString(sapApiUser)).toString();
		System.out.println(authorizationStr);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectPrimaryListByName(String name) throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectPrimaryListByName", name);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectPrimaryListByResult(String result) throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectPrimaryListByResult", result);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectPrimaryListByDelFlag(Integer delFlag) throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectPrimaryListByDelFlag", delFlag);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectPrimaryListByDepartIdAndLevel(String departId) throws Exception {
		SapOrg sapOrg = sapOrgService.selectById(departId);
		if (StringUtils.isBlank(departId) || WebConstants.TOP_DEPART.toString().equals(departId)) {
			departId = WebConstants.TOP_DEPART.toString();
			sapOrg = SapOrg.builder().id(WebConstants.TOP_DEPART.toString()).sapName(WebConstants.TOP_DEPART_NAME)
					.level("1").build();
		}
		PageData pd = new PageData();
		pd.put("departId", departId);
		pd.put("level", sapOrg.getLevel());
		return (List<SapUser>) findList("SapUserMapper.selectPrimaryListByDepartIdAndLevel", pd);
	}

	public SapUser toInfoDetail(SapUser sapUser) throws Exception {
		if (sapUser != null) {
			List<SapUser> users = selectListByPersonId(sapUser.getPersonId());
			List<List<SapOrg>> departList = Lists.newArrayList();
			for (int i = 0; i < users.size(); i++) {
				SapUser user = users.get(i);
				List<SapOrg> orgs = Lists.newArrayList();
				if (Tools.notEmpty(user.getFirstOrgId())) {
					orgs.add(sapOrgService.selectById(user.getFirstOrgId()));
				}
				if (Tools.notEmpty(user.getSecondOrgId())) {
					orgs.add(sapOrgService.selectById(user.getSecondOrgId()));
				}
				if (Tools.notEmpty(user.getThirdOrgId())) {
					orgs.add(sapOrgService.selectById(user.getThirdOrgId()));
				}
				if (Tools.notEmpty(user.getFourthOrgId())) {
					orgs.add(sapOrgService.selectById(user.getFourthOrgId()));
				}
				departList.add(orgs);
			}
			sapUser.setDepartList(departList);
		}
		return sapUser;
	}

	public List<SapUser> toInfoDetails(List<SapUser> sapUsers) throws Exception {
		for (SapUser sapUser : sapUsers) {
			toInfoDetail(sapUser);
		}
		return sapUsers;
	}

	public SapUser getDetailById(String userId) throws Exception {
		SapUser sapUser = selectByUserId(userId);
		return toInfoDetail(sapUser);
	}

	// 更新用户头像
	public void updateAvatars() throws Exception {
		List<SapUser> successUsers = selectPrimaryListByResult(SapUserResultEnum.SUCCESS.getCode());
		for (SapUser sapUser : successUsers) {
			WxCpUser user = WXCpUserUtil.getByIdOrEmail(sapUser.getPersonId(), sapUser.getEmail());
			if (user != null && StringUtils.isNotBlank(user.getAvatar())) {
				sapUser.setAvatar(user.getAvatar());
				update("SapUserMapper.updateAvatar", sapUser);
			}
		}
	}

	public void updateResult(SapUser sapUser) throws Exception {
		update("SapUserMapper.updateResult", sapUser);
	}

	public void deleteQuitUserByUserId(String userId, String oprUserId) throws Exception {
		SapUser sapUser = selectByUserId(userId);
		String personId = sapUser.getPersonId();
		List<SapUser> sapUsers = selectListByPersonId(personId);
		for (SapUser user : sapUsers) {
			updateDelFlag(user.getUserId(), DelFlagEnum.DELETED);
		}
		WXCpUserUtil.delete(personId);
		sapOprLogService.saveOne(JSONUtil.toString(sapUsers), WebConstants.OPRTYPE_DELETE, oprUserId);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectToBeConfirmedPrimaryList(GetToBeConfirmedListDTO dto) throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectToBeConfirmedPrimaryList", dto);
	}

	public void updateIngore(String userId) {
	}

	public void updateSync(String userId) {
	}
}
