package org.fouram.service;

import java.util.List;
import java.util.Map;

import org.fouram.core.base.service.BaseService;
import org.fouram.core.plugin.weixin.cp.util.WXCpUserUtil;
import org.fouram.entity.WxDepart;
import org.fouram.entity.WxUser;
import org.fouram.entity.WxUserDepart;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.collect.Maps;

import me.chanjar.weixin.cp.bean.WxCpUser;

@Service
public class WxUserService extends BaseService {

	@Autowired
	private WxDepartService wxDepartService;

	private Map<Long, List<WxDepart>> userDepartMap = Maps.newHashMap();

	public void saveWxUsers() throws Exception {
		update("WxUserMapper.updateAllDelete", null);

		List<WxCpUser> users = WXCpUserUtil.listAllUser();
		List<WxDepart> departs = null;
		WxUser wxUser = null;
		WxUser dbUser = null;
		String userId = null;
		WxUserDepart wxUserDepart = null;
		WxUserDepart dbUserDepart = null;
		for (WxCpUser user : users) {
			userId = user.getUserId();
			wxUser = WxUser.builder().userId(userId).email(user.getEmail()).name(user.getName())
					.departId(user.getDepartIds()[0]).build();
			departs = getUserDepart(wxUser.getDepartId());
			if (departs.size() > 0) {
				wxUser.setFirstDepartName(departs.get(0).getName());
				wxUser.setFirstDepartId(departs.get(0).getId());
			}
			if (departs.size() > 1) {
				wxUser.setSecondDepartName(departs.get(1).getName());
				wxUser.setSecondDepartId(departs.get(1).getId());
			}
			wxUser.setPosition(user.getPosition());
			dbUser = selectById(userId);
			if (dbUser == null) {
				save("WxUserMapper.save", wxUser);
			} else {
				wxUser.setUserId(dbUser.getUserId());
				update("WxUserMapper.update", wxUser);
			}

			for (Long departId : user.getDepartIds()) {
				wxUserDepart = WxUserDepart.builder().departId(departId).userId(userId).build();
				departs = getUserDepart(departId);
				if (departs.size() > 0) {
					wxUserDepart.setFirstDepartName(departs.get(0).getName());
					wxUserDepart.setFirstDepartId(departs.get(0).getId());
				}
				if (departs.size() > 1) {
					wxUserDepart.setSecondDepartName(departs.get(1).getName());
					wxUserDepart.setSecondDepartId(departs.get(1).getId());
				}
				if (departs.size() > 2) {
					wxUserDepart.setThirdDepartName(departs.get(2).getName());
					wxUserDepart.setThirdDepartId(departs.get(2).getId());
				}
				if (departs.size() > 3) {
					wxUserDepart.setFourthDepartName(departs.get(3).getName());
					wxUserDepart.setFourthDepartId(departs.get(3).getId());
				}
				dbUserDepart = selectWxUserDepartByUserIdAndDepartId(userId, departId);
				if (dbUserDepart == null) {
					save("WxUserMapper.saveWxUserDepart", wxUserDepart);
				} else {
					update("WxUserMapper.updateWxUserDepart", wxUserDepart);
				}
			}
		}
	}

	public WxUserDepart selectWxUserDepartByUserIdAndDepartId(String userId, Long departId) throws Exception {
		WxUserDepart wxUserDepart = WxUserDepart.builder().userId(userId).departId(departId).build();
		return (WxUserDepart) findObject("WxUserMapper.selectWxUserDepartByUserIdAndDepartId", wxUserDepart);
	}
	
	@SuppressWarnings("unchecked")
	public List<WxUserDepart> selectWxUserDepartsByUserId(String userId) throws Exception {
		return (List<WxUserDepart>) findList("WxUserMapper.selectWxUserDepartsByUserId", userId);
	}

	public WxUser selectById(String userId) throws Exception {
		return (WxUser) findObject("WxUserMapper.selectById", userId);
	}

	private List<WxDepart> getUserDepart(Long departId) throws Exception {
		if (!userDepartMap.containsKey(departId)) {
			userDepartMap.put(departId, wxDepartService.findParentList(departId));
		}
		return userDepartMap.get(departId);
	}

	@SuppressWarnings("unchecked")
	public List<String> selectAllUserIds() throws Exception {
		return (List<String>) findList("WxUserMapper.selectAllUserIds", null);
	}

	public String getFullDepartName(String userId) throws Exception {
		List<WxUserDepart> wxUserDeparts = selectWxUserDepartsByUserId(userId);
		StringBuffer sb = new StringBuffer();
		for(WxUserDepart wxUserDepart : wxUserDeparts) {
			if(sb.indexOf(wxUserDepart.getFullDepartName()) < 0) {
				sb.append(wxUserDepart.getFullDepartName()).append(";");
			}
		}
		return sb.substring(0, sb.length() - 1);
	}
}
